/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2020 by TOPPERS PROJECT Educational Working Group.
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: lcd_text.c 2416 2020-05-13 11:00:31Z roi $
 */

/* 
 *  LCDץ
 */

#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include <stdio.h>
#include <string.h>
#include <target_syssvc.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "device.h"
#include "spi.h"
#include "pinmode.h"
#include "adafruit_st7789.h"
//#include "Fonts.h"
#include "glcd_disp.h"
#include "lcd_text.h"

/*
 *  ӥΥ顼Υ
 */
Inline void
svc_perror(const char *file, int_t line, const char *expr, ER ercd)
{
	if (ercd < 0) {
		t_perror(LOG_ERROR, file, line, expr, ercd);
	}
}

#define	SVC_PERROR(expr)	svc_perror(__FILE__, __LINE__, #expr, (expr))

#define LCD_WIDTH    240
#define LCD_HEIGHT   240

/*
 *  LCDǡΰ
 */

LCD_Handler_t  LcdHandle;
LCD_DrawProp_t DrawProp;

/*
 *  SW1
 */
void sw_int(void)
{
	syslog_0(LOG_NOTICE, "## sw_int() ##");
}

/*
 *  GPIOؿ
 */
static void
pinMode(uint8_t no, uint32_t mode)
{
	Arduino_PortControlBlock *pgcb = getGpioTable(DIGITAL_PIN, no);
	GPIO_Init_t GPIO_Init_Data;

	if(pgcb == NULL)
		return;
	pinClock(no);
	GPIO_Init_Data.mode  = mode;
	GPIO_Init_Data.pull  = GPIO_PULLMODE;
	GPIO_Init_Data.otype = GPIO_OTYPE;
	GPIO_Init_Data.speed = GPIO_SPEED_FAST;
	gpio_setup(pgcb->giobase, &GPIO_Init_Data, pgcb->giopin);
}

#ifndef _GLCD_DISP_H_
/*
 *  ʸ
 *  param1  x0: ʸ Xɸ
 *  param2  y0: ʸ Yɸ
 *  param3  c:  ʸ᡼ؤΥݥ
 */
static void
DrawChar(uint16_t x0, uint16_t y0, const uint8_t *c)
{
	LCD_Handler_t *hlcd = DrawProp.hlcd;
	sFONT    *pFont = DrawProp.pFont;
	uint32_t h, w, stride;
	uint8_t  *pchar;

	stride = ((pFont->Width + 7)/8);
	lcd_setAddrWindow(hlcd, x0, y0, x0 + pFont->Width - 1, y0 + pFont->Height - 1);

	for(h = 0 ; h < pFont->Height ; h++){
		pchar = ((uint8_t *)c + stride * h);
		for(w = 0; w < pFont->Width ; w++){
			if(pchar[w/8] & (1 << (7 - (w % 8)))){
				lcd_writedata2(hlcd, DrawProp.TextColor, 1);
			}
			else{
				lcd_writedata2(hlcd, DrawProp.BackColor, 1);
			}
		}
	}
}

/*
 *  ʸ
 *  param1  x0: ʸ Xɸ
 *  param2  y0: ʸ Yɸ
 *  param3  襢(ϰϤ0x200x7Eޤ)
 */
void
lcd_DisplayChar(LCD_DrawProp_t *pDrawProp, uint16_t x0, uint16_t y0, uint8_t Ascii)
{
	sFONT    *pFont = pDrawProp->pFont;
	DrawChar(x0, y0, &pFont->table[(Ascii-' ') * pFont->Height * ((pFont->Width + 7) / 8)]);
}

/*
 *  ʸ
 *  param1  x0: ʸ Xɸ
 *  param2  y0: ʸ Yɸ
 *  param3  Text: ʸؤΥݥ
 */
void
lcd_DisplayString(LCD_DrawProp_t *pDrawProp, uint16_t x0, uint16_t y0, char *Text)
{
	sFONT	*pFont = pDrawProp -> pFont;
	uint16_t ref_column =1,i=0;
	
	/* ϥ֤򥹥꡼˼*/
	ref_column = x0;
	if((ref_column <1) || (ref_column >= 0x8000)){
			ref_column = 1;
	}
	/* ʸȤäʸ褹 */
	while((*Text != 0) && (((pDrawProp->hlcd->_width - (i*pFont->Width)) & 0xFFFF) >= pFont->Width)){
		lcd_DisplayChar(pDrawProp,ref_column,y0,*Text);
		ref_column += pFont->Width;	/* Υ˰ư */
		Text++;						/* ʸ˰ư */
		i++;
	}
}
#endif


/*
 *  ᥤ󥿥 220606
 */
void
main_task(intptr_t exinf)
{
	SPI_Init_t spi_initd;
	SPI_Handle_t  *hspi;
	LCD_Handler_t *hlcd;
	ER_UINT	ercd;
//	char *ptr = "Hello World !!!!!!";220902

//	int idata = 1234;
//	float fdata = 3.1f;

//char buf[32];
//sprintf(buf,"%g", fdata);
//	sprintf(buf,"%d", idata);
//printf("%s\n",buf);
	
float fdata;
	fdata = 3.14*100;
int idata;
	idata = (int)fdata;
	
char buf[24];
sprintf(buf,"%d", idata);	
printf("%d\n",idata);
	
	
	SVC_PERROR(syslog_msk_log(LOG_UPTO(LOG_INFO), LOG_UPTO(LOG_EMERG)));
	syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (int_t) exinf);

	/*
	 *  ꥢݡȤν
	 *
	 *  ƥƱꥢݡȤȤʤɡꥢ
	 *  ݡȤץѤߤξˤϤE_OBJ顼ˤʤ뤬پ
	 *  ʤ
	 */
	ercd = serial_opn_por(TASK_PORTID);
	if (ercd < 0 && MERCD(ercd) != E_OBJ) {
		syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
									itron_strerror(ercd), SERCD(ercd));
	}
	SVC_PERROR(serial_ctl_por(TASK_PORTID,
							(IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

	syslog_0(LOG_NOTICE, "LCD TEST START");
	/*
	 *  LCD-CS PORT(D10-PB6)
	 */
	pinMode(CS_PORT, GPIO_MODE_OUTPUT);
	/*
	 *  RS PORT(D8-PA9)
	 */
	pinMode(RS_PORT, GPIO_MODE_OUTPUT);
	/*
	 *  RST PORT(D9-PC7)
	 */
	pinMode(RST_PORT, GPIO_MODE_OUTPUT);
	/*
	 *  CS2 PORT(D4-PB5)
	 */
	pinMode(CS2_PORT, GPIO_MODE_OUTPUT);

	spi_initd.Prescaler     = SPI_BAUDRATEPRESCALER_32;
	spi_initd.Direction     = SPI_DIRECTION_2LINES;
	spi_initd.CLKPhase      = SPI_PHASE_1EDGE;
	spi_initd.CLKPolarity   = SPI_POLARITY_LOW;
	spi_initd.CRC           = SPI_CRC_DISABLE;
	spi_initd.CRCPolynomial = 7;
	spi_initd.DataSize      = SPI_DATASIZE_8BIT;
	spi_initd.SignBit       = SPI_DATA_MSB;
	spi_initd.NSS           = SPI_NSS_SOFT;
	spi_initd.TIMode        = SPI_TIMODE_DISABLE;
	spi_initd.Mode = SPI_MODE_MASTER;
	spi_initd.semid = SPI1DMA_SEM;
	spi_initd.semlock = 0;

	if((hspi = spi_init(SPI1_PORTID, &spi_initd)) == NULL){
		syslog_0(LOG_ERROR, "spi_start initialize error !");
		slp_tsk();
	}

	hlcd = &LcdHandle;
	hlcd->hspi = hspi;
	hlcd->rotation = 0;
	hlcd->spi_lock= SPI1LOCK_SEM;
	hlcd->cs_pin  = CS_PORT;
	hlcd->rs_pin  = RS_PORT;
	hlcd->rst_pin = RST_PORT;
	hlcd->cs2_pin = CS2_PORT;
	DrawProp.hlcd = hlcd;
	lcd_init(hlcd, LCD_WIDTH, LCD_HEIGHT);
	DrawProp.BackColor = ST7789_BLACK;
	DrawProp.TextColor = ST7789_WHITE;

	lcd_fillRect(hlcd, 0, 0,  hlcd->_width, hlcd->_height, DrawProp.BackColor);
	dly_tsk(1000);
	lcd_drawPixel(hlcd, hlcd->_width/2, hlcd->_height/2, ST7789_GREEN);
	dly_tsk(1000);

	DrawProp.TextColor = ST7789_BLACK;
	DrawProp.BackColor = ST7789_WHITE;
	lcd_fillRect(hlcd, 0, 0,  hlcd->_width, hlcd->_height, DrawProp.BackColor);

	DrawProp.pFont = &Font16;
	lcd_DisplayStringAt(&DrawProp, 0, 80,(uint8_t *)"Refernce Signal",CENTER_MODE); //0902
	lcd_DisplayStringAt(&DrawProp, 0, 90, (uint8_t *)buf,CENTER_MODE); //0902
	syslog_0(LOG_NOTICE, buf); //0826

	DrawProp.TextColor = ST7789_BLUE;
	DrawProp.pFont = &Font12;
	lcd_DisplayStringAt(&DrawProp, 0, 110, (uint8_t *)"Feedback Signal",CENTER_MODE);//0902

	syslog_0(LOG_NOTICE, "LCD TEST END");
	slp_tsk();
	spi_deinit(hspi);

	syslog(LOG_NOTICE, "Sample program ends.");
//	SVC_PERROR(ext_ker());
}









float  vmcom, vscom;/*220902*/


# if 0
/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: adctest.c 2416 2017-05-16 17:58:20Z roi $
 */

/* 
 *  ADCƥȤ
 */

//#include <kernel.h>
//#include <t_syslog.h>
//#include <t_stdlib.h>
//#include <string.h>
#include <sil.h>
//#include "syssvc/serial.h"
//#include "syssvc/syslog.h"
//#include "kernel_cfg.h"
#include <target_syssvc.h>
//#include "device.h"
#include "adctest.h"

/*
 *  ӥΥ顼Υ
 */
Inline void
svc_perror(const char *file, int_t line, const char *expr, ER ercd)
{
	if (ercd < 0) {
		t_perror(LOG_ERROR, file, line, expr, ercd);
	}
}

#define	SVC_PERROR(expr)	svc_perror(__FILE__, __LINE__, #expr, (expr))


#define ADC1_CLOCK   (SysFrePCLK2/2)
#define SAMPLE_CLOCK (ADC1_CLOCK/480)
#define NUM_ADCBUF_M   (SAMPLE_CLOCK*10/1000)		/* 10ms */
#define NUM_ADCBUF_S   (SAMPLE_CLOCK*10/1000)		/* 10ms 220104 */


volatile uint32_t uhADCxConvertedValue_M[NUM_ADCBUF_M];
volatile uint32_t uhADCxConvertedValue_S[NUM_ADCBUF_S];		/* 10ms 220104 */


/*
 *  SW1
 */
void sw_int(void)
{
	syslog_0(LOG_NOTICE, "## sw_int() ##");
}

/*
 *  ADCžλХåؿ  10ms 220104
 */
static void ADC_ConvCpltCallback2(ADC_Handle_t* hadc)
{
	syslog_4(LOG_NOTICE, "0(%d) 1(%d) 2(%d) 3(%d)",uhADCxConvertedValue_M[0], uhADCxConvertedValue_M[1], uhADCxConvertedValue_M[2], uhADCxConvertedValue_M[3]);
	syslog_4(LOG_NOTICE, "0(%d) 1(%d) 2(%d) 3(%d)",uhADCxConvertedValue_S[0], uhADCxConvertedValue_S[1], uhADCxConvertedValue_S[2], uhADCxConvertedValue_S[3]);
	SVC_PERROR(isig_sem(ADCDMA_SEM));
}

float  vmcom, vscom;

/*
 *  ᥤ󥿥
 */
#define LED1_INTERVAL 1000
void mainad_task(intptr_t exinf)
{
	ADC_Init_t     aInit;
	ADC_ChannelConf_t sConfig;
	ADC_Handle_t *hadc;
	ER_UINT	ercd;
	SYSTIM tim1, tim2;
	float  v, sum;
	int    i, j;
	int    tmp1, tmp2;

	SVC_PERROR(syslog_msk_log(LOG_UPTO(LOG_INFO), LOG_UPTO(LOG_EMERG)));
	syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (int_t) exinf);

	/*
	 *  ꥢݡȤν
	 *
	 *  ƥƱꥢݡȤȤʤɡꥢ
	 *  ݡȤץѤߤξˤϤE_OBJ顼ˤʤ뤬پ
	 *  ʤ
	 */
	ercd = serial_opn_por(TASK_PORTID);
	if (ercd < 0 && MERCD(ercd) != E_OBJ) {
		syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
									itron_strerror(ercd), SERCD(ercd));
	}
	SVC_PERROR(serial_ctl_por(TASK_PORTID,
							(IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

	/*
	 *  ADC1
	 */
	aInit.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV2;
	aInit.Resolution = ADC_RESOLUTION_12B;
	aInit.ScanConvMode = ADC_SCANMODE_DISABLE;
	aInit.ContinuousConvMode = ADC_CONTINUOUS_ENABLE;
	aInit.DiscontinuousConvMode = ADC_DISCONTINUOUS_DISABLE;
	aInit.NumDiscConversion = 0;
	aInit.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
	aInit.ExternalTrigConv = ADC_EXTERNALTRIGCONV_T1_CC1;
	aInit.DataAlign = ADC_DATAALIGN_RIGHT;
	aInit.NumConversion = 1;
	aInit.DMAContinuousRequests = ADC_DMACONTINUOUS_ENABLE2;
	aInit.EOCSelection = ADC_EOC_SEQ_DISABLE;

	if((hadc = adc_init(ADC1_PORTID, &aInit)) == NULL){
		/* Initialization Error */
		syslog_0(LOG_NOTICE, "## adc_init ERROR ##");
	}
//	hadc->xfercallback = ADC_ConvCpltCallback2;

	/*
	 *  CHANNELADC1-8
	 */
	sConfig.Channel = ADCx_CHANNEL_MAIN;
	sConfig.Rank = 1;
	sConfig.GpioBase = ADCx_CHANNEL_GPIO_PORT_MAIN;
	sConfig.GpioPin  = ADCx_CHANNEL_PINNO_MAIN;
//  sConfig.SamplingTime = ADC_SAMPLETIME_3CYCLES;
	sConfig.SamplingTime = ADC_SAMPLETIME_480CYCLES;
	if(adc_setupchannel(hadc, &sConfig) != E_OK){
		/* Channel Configuration Error */
		syslog_0(LOG_NOTICE, "## adc_setupchannel ERROR ##");
	}
	
	for(i = 0 ; i < 1000 ; i++){
/*
	 *  CHANNELADC1-8
	 */
	sConfig.Channel = ADCx_CHANNEL_MAIN;
	sConfig.Rank = 1;
	sConfig.GpioBase = ADCx_CHANNEL_GPIO_PORT_MAIN;
	sConfig.GpioPin  = ADCx_CHANNEL_PINNO_MAIN;
//  sConfig.SamplingTime = ADC_SAMPLETIME_3CYCLES;
	sConfig.SamplingTime = ADC_SAMPLETIME_480CYCLES;
	if(adc_setupchannel(hadc, &sConfig) != E_OK){
		/* Channel Configuration Error */
		syslog_0(LOG_NOTICE, "## adc_setupchannel ERROR ##");
	}		
		
		
		
		
		
		
		
		
		
		memset(uhADCxConvertedValue_M, 0, NUM_ADCBUF_M*4);
		get_tim(&tim1);
		/*
		 * DMAꡧDMANUM_ADCBUFʬ
		 */
		if(adc_start_dma(hadc, (uint32_t*)&uhADCxConvertedValue_M[0], NUM_ADCBUF_M) != E_OK){
			/* Start Conversation Error */
			syslog_0(LOG_NOTICE, "## adc_start_dma ERROR ##");
		}
		while(hadc->status == ADC_STATUS_BUSY){
			twai_sem(ADCDMA_SEM, 10);
		}
		get_tim(&tim2);
		
dly_tsk(LED1_INTERVAL);							/*220106*/
		twai_sem(ADCDMA_SEM, 10);				/*220105*/
		adc_end_dma(hadc);
		for(j = 0, sum = 0.0 ; j < NUM_ADCBUF_M ; j++)
			sum += uhADCxConvertedValue_M[j];
		sum /= NUM_ADCBUF_M;
		v = (sum * 3.3) / 4096.0;
		vmcom = v;								/*220719*/
		tmp1 = (int)v;
		if(v < 0.0)
			tmp2 = (tmp1 - v) * 100;
		else
			tmp2 = (v - tmp1) * 100;
		syslog_5(LOG_NOTICE, "%d:dma time(%d) sum(%d) vm(%d.%02d)", i, tim2-tim1, (int)sum, tmp1, tmp2);
		dly_tsk(100);
		hadc->xfercallback = ADC_ConvCpltCallback2;	/*220105release*/
dly_tsk(LED1_INTERVAL);								/*220106*/
	}
	syslog_2(LOG_NOTICE, "## STOP[%08x] (%d) ##", &uhADCxConvertedValue_M, hadc->status);
	slp_tsk();
//	SVC_PERROR(ext_ker());
}


/*
 *  ֥
 */
#define LED2_INTERVAL 1000							/*220105*/
void sub_task(intptr_t exinf)
{
	ADC_Init_t     aInit;
	ADC_ChannelConf_t sConfigs;
	ADC_Handle_t *hadc;
	ER_UINT	ercd;
	SYSTIM tim1, tim2;
	float  vs, sums;
	int    i, j;
	int    tmp1s, tmp2s;

	SVC_PERROR(syslog_msk_log(LOG_UPTO(LOG_INFO), LOG_UPTO(LOG_EMERG)));
	syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (int_t) exinf);

	/*
	 *  ꥢݡȤν
	 *
	 *  ƥƱꥢݡȤȤʤɡꥢ
	 *  ݡȤץѤߤξˤϤE_OBJ顼ˤʤ뤬پ
	 *  ʤ
	 */
	ercd = serial_opn_por(TASK_PORTID);
	if (ercd < 0 && MERCD(ercd) != E_OBJ) {
		syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
									itron_strerror(ercd), SERCD(ercd));
	}
	SVC_PERROR(serial_ctl_por(TASK_PORTID,
							(IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

	/*
	 *  ADC1
	 */
	aInit.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV2;
	aInit.Resolution = ADC_RESOLUTION_12B;
	aInit.ScanConvMode = ADC_SCANMODE_DISABLE;
	aInit.ContinuousConvMode = ADC_CONTINUOUS_ENABLE;
	aInit.DiscontinuousConvMode = ADC_DISCONTINUOUS_DISABLE;
	aInit.NumDiscConversion = 0;
	aInit.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
	aInit.ExternalTrigConv = ADC_EXTERNALTRIGCONV_T1_CC1;
	aInit.DataAlign = ADC_DATAALIGN_RIGHT;
	aInit.NumConversion = 1;
	aInit.DMAContinuousRequests = ADC_DMACONTINUOUS_ENABLE2;
	aInit.EOCSelection = ADC_EOC_SEQ_DISABLE;

	if((hadc = adc_init(ADC1_PORTID, &aInit)) == NULL){
		/* Initialization Error */
		syslog_0(LOG_NOTICE, "## adc_init ERROR ##");
	}
//	hadc->xfercallback = ADC_ConvCpltCallback2;

	/*
	 *  CHANNELADC1-4
	 */
	sConfigs.Channel = ADCx_CHANNEL_SUB;
	sConfigs.Rank = 1;
	sConfigs.GpioBase = ADCx_CHANNEL_GPIO_PORT_SUB;
	sConfigs.GpioPin  = ADCx_CHANNEL_PINNO_SUB;
//  sConfigs.SamplingTime = ADC_SAMPLETIME_3CYCLES;
	sConfigs.SamplingTime = ADC_SAMPLETIME_480CYCLES;
	if(adc_setupchannel(hadc, &sConfigs) != E_OK){
		/* Channel Configuration Error */
		syslog_0(LOG_NOTICE, "## adc_setupchannel ERROR ##");
	}
	
	for(i = 0 ; i < 1000 ; i++){
	/*
	 *  CHANNELADC1-4
	 */
	sConfigs.Channel = ADCx_CHANNEL_SUB;
	sConfigs.Rank = 1;
	sConfigs.GpioBase = ADCx_CHANNEL_GPIO_PORT_SUB;
	sConfigs.GpioPin  = ADCx_CHANNEL_PINNO_SUB;
//  sConfigs.SamplingTime = ADC_SAMPLETIME_3CYCLES;
	sConfigs.SamplingTime = ADC_SAMPLETIME_480CYCLES;
	if(adc_setupchannel(hadc, &sConfigs) != E_OK){
		/* Channel Configuration Error */
		syslog_0(LOG_NOTICE, "## adc_setupchannel ERROR ##");
	}		
		
		
		
		
		memset(uhADCxConvertedValue_S, 0, NUM_ADCBUF_S*4);
		get_tim(&tim1);
		/*
		 * DMAꡧDMANUM_ADCBUFʬ
		 */
		if(adc_start_dma(hadc, (uint32_t*)&uhADCxConvertedValue_S[0], NUM_ADCBUF_S) != E_OK){
			/* Start Conversation Error */
			syslog_0(LOG_NOTICE, "## adc_start_dma ERROR ##");
		}
		while(hadc->status == ADC_STATUS_BUSY){
			twai_sem(ADCDMA_SEM, 10);
		}
		get_tim(&tim2);

dly_tsk(LED2_INTERVAL);								/*220106*/
		twai_sem(ADCDMA_SEM, 10);					/*220105*/
		adc_end_dma(hadc);
		for(j = 0, sums = 0.0 ; j < NUM_ADCBUF_S ; j++)
			sums += uhADCxConvertedValue_S[j];
		sums /= NUM_ADCBUF_S;
		vs = (sums * 3.3) / 4096.0;
		vscom = vs;								/*220719*/
		tmp1s = (int)vs;
		if(vs < 0.0)
			tmp2s = (tmp1s - vs) * 100;
		else
			tmp2s = (vs - tmp1s) * 100;
		syslog_5(LOG_NOTICE, "%d:dma time(%d) sums(%d) vs(%d.%02d)", i, tim2-tim1, (int)sums, tmp1s, tmp2s);
		dly_tsk(100);
		hadc->xfercallback = ADC_ConvCpltCallback2;	/*220105release*/
dly_tsk(LED2_INTERVAL);								/*220106*/
	}
	syslog_2(LOG_NOTICE, "## STOP[%08x] (%d) ##", &uhADCxConvertedValue_S, hadc->status);
	slp_tsk();
//	SVC_PERROR(ext_ker());}
}
#endif



/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: pwmtest.c 2416 2012-09-07 08:06:20Z ertl-hiro $
 */

/* 
 *  PWMƥȤ
 *  TIM1-CH1(PA8;D750%),TIM1-CH2(PA9:D8 75%)LED³ƥƥ
 */

#if 0	/* delete lines */
#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include <target_syssvc.h>
#include <sil.h>
#include "kernel_cfg.h"
#include "device.h"
#endif	/* delete lines */
#include "pwm.h"
#include "pwmtest.h"

#define sil_orw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) | (b))

#if 0	/* delete lines */
/*
 *  ӥΥ顼Υ
 */
Inline void
svc_perror(const char *file, int_t line, const char *expr, ER ercd)
{
	if (ercd < 0) {
		t_perror(LOG_ERROR, file, line, expr, ercd);
	}
}

#define	SVC_PERROR(expr)	svc_perror(__FILE__, __LINE__, #expr, (expr))
#endif	/* delete lines */

/* USER CODE BEGIN Private defines */
/* Compute the prescaler value to have TIM1 counter clock equal to 10000 Hz */

#define PRESCALER_VALUE     (uint32_t)(((SystemFrequency) / 10000) - 1)

/* Initialize TIMx peripheral as follows:
   + Prescaler = (SystemCoreClock / 10000) - 1
   + Period = (101 - 1)
   + ClockDivision = 0
   + Counter direction = Up
*/
#define PERIOD_VALUE        (uint32_t)(101 - 1)  /* Period Value  */

static TIM_Handle_t htim1;

#if 0	/* delete lines */
/*
 *  SW1
 */
void sw_int(void)
{
	syslog_0(LOG_NOTICE, "## sw_int() ##");
}
#endif	/* delete lines */


/*
 *  ᥤ󥿥
 */
void mainp_task(intptr_t exinf)
{
	TIM_OC_Init_t sConfigOC = {0};
	ER_UINT	ercd;
	int i, pulse;
	float ferror,fi,fp,fpwm,fi_old;

	SVC_PERROR(syslog_msk_log(LOG_UPTO(LOG_INFO), LOG_UPTO(LOG_EMERG)));
	syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (int_t) exinf);

	/*
	 *  ꥢݡȤν
	 *
	 *  ƥƱꥢݡȤȤʤɡꥢ
	 *  ݡȤץѤߤξˤϤE_OBJ顼ˤʤ뤬پ
	 *  ʤ
	 */
	ercd = serial_opn_por(TASK_PORTID);
	if (ercd < 0 && MERCD(ercd) != E_OBJ) {
		syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
									itron_strerror(ercd), SERCD(ercd));
	}
	SVC_PERROR(serial_ctl_por(TASK_PORTID,
							(IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

	htim1.Init.Prescaler = PRESCALER_VALUE;
	htim1.Init.CounterMode = TIM_COUNTERMODE_UP;
	htim1.Init.Period = PERIOD_VALUE;
	htim1.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
	htim1.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
	ercd = pwm_init(TIM1_PORTID, &htim1);
	if(ercd != E_OK){
		syslog_1(LOG_ERROR, "TIM1 init error(%d) !", ercd);
		slp_tsk();
	}
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_AHB1ENR), RCC_AHB1ENR_GPIOAEN);
	pwm_gpio_channel(&htim1, TIM_CHANNEL1_ID, TADR_GPIOA_BASE, TC1_IN1_PIN);
	pwm_gpio_channel(&htim1, TIM_CHANNEL2_ID, TADR_GPIOA_BASE, TC1_IN2_PIN);

	sConfigOC.OCMode = TIM_OCMODE_PWM1;
	sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
	sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
	pwm_stop(&htim1, TIM_CHANNEL1_ID);
	
// 20220719added	


	fi_old =0;
	for(;;){
		pwm_stop(&htim1, TIM_CHANNEL1_ID);
		twai_sem(ADCDMA_SEM, 10);					/*220803*/
		ferror = vmcom - vscom;
		fp= ferror * 0.8;
		fi = ferror * 0.01 + fi_old;
		fpwm = fp + fi;
		fi_old = fi;
		sConfigOC.Pulse  = (fpwm/3.3)*100;
		if(sConfigOC.Pulse > 100){
			sConfigOC.Pulse = 100;
		}else if(sConfigOC.Pulse < 0){
			sConfigOC.Pulse = 0;
		}
		ercd = pwm_start(&htim1, &sConfigOC, TIM_CHANNEL1_ID);
		if(ercd != E_OK){
			syslog_1(LOG_ERROR, "TIM1 channel1 error(%d) !", ercd);
			slp_tsk();
		}
		dly_tsk(60);
	}
	
	pwm_stop(&htim1, TIM_CHANNEL1_ID);
	pwm_stop(&htim1, TIM_CHANNEL2_ID);
	syslog_0(LOG_NOTICE, "STOP");
	slp_tsk();

	syslog(LOG_NOTICE, "Sample program ends.");
//	SVC_PERROR(ext_ker());
}


